/*  ESDOutput.cpp - ESD output plugin
 *  Copyright (C) 1999 Andy Lo A Foe <andy@alsa-project.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/ 

#include "../config.h"
#ifdef USE_ESD
#include <stdio.h>
#include <esd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/mman.h>
#include <unistd.h>
#include <dlfcn.h>
#include "output_plugin.h"

int esd_socket;

int esd_init()
{
	esd_socket = -1;
	char *host = NULL;
	char *name = NULL;
	esd_format_t format = ESD_BITS16 | ESD_STEREO | ESD_STREAM | ESD_PLAY;
	int rate = 44100;
                
	printf("loading libesd.so\n");
	void *handle = dlopen("libesd.so", RTLD_LAZY);
	if (handle == NULL) {
		printf("error: %s\n", dlerror());
		return 0;
	} else {
		int (*esd_play_stream_fallback)(esd_format_t,int,
                        const char *, const char *);
                        
                 esd_play_stream_fallback = dlsym(handle, "esd_play_stream_fallback");
                 esd_socket = (*esd_play_stream_fallback)(format, rate, host,
                                name);
                if (esd_socket < 0) {
					dlclose(handle);
					return 0; 
                }
	}
	return 1;
}

int esd_open(int card, int device)
{
	int err;

	if (esd_socket > 0) 
		return 1;
	return 0;
}


void esd_close()
{
	return;
}


int esd_write(void *data, int count)
{
	write(esd_socket, data, count);
	return 1;
}


int esd_set_buffer(int fragment_size, int fragment_count)
{
	printf("ESD: fragments fixed at 4096/3\n");
	return 1;
}


int esd_set_sample_rate(int rate)
{
	printf("ESD: rate fixed at 44100Hz\n");
	return 1;
}


output_plugin esd_output = {
	{ "ESD output v1.0" },
	{ "Andy Lo A Foe" },
	esd_init,
	esd_open,
	esd_close,
	esd_write,
	esd_set_buffer,
	esd_set_sample_rate,
	NULL
};
#endif
